#!/bin/bash
# (c) Joe Linux @ wwww.linuxrouen.fr / under GNU General Public License v.3
# Last Update: 2022/07/24

# This is a GNU/Linux Bash Script for "All-RAM Data" (MIDI SysEx Bulk Data Dump):
#   - SAVE/LOAD between Yamaha Electone 'Vintage' Series organs (1986-to-2001) & PCs,
#   - BACKUP between Yamaha MDR 'Vintage' Peripherals and PCs,
#       . Then, these MDR Backups can be sent from PC to Electone, and
#   - MONITOR MIDI Traffic (SysEx/Readable Data & Messages) on the selected MIDI/USB I/F.

# To run YesXR.sh (Yamaha Electone Simple SysEX Recorder) under GNU/Linux distributions:
#   - the YAD v. =0.40.x or >=10.1 -and- ALSA-UTILS v. >=1.2.2 packages, as well as
#   - the GNOME-TERMINAL v. >=3.36.2 package (only needed for the MIDI Monitor)
#       MUST be installed on your GNU/Linux operating system.
# This script's GUI is using (as much as possible) the Theme of the Desktop environment.
# Note: The YesXR Terminal can be, if necessary, used for debugging & testing purpose.

#=====================================#
# Important Notes:
#   YAD's Options which are +/-incompatible across versions and distributions:
#   --width & height: The displayed result can vary vs the used Distribution / DE.
#   --file-selection: File selection windows could be Not at all displayed.
#   --fixed: Windows disappear after 2nd and more levels or other behaviors.
#   -- progress --pulsate: Different behaviors across distributions.
#===========================#

#=============================================================================#
#========================> YESXR SCRIPT FLOW CHART <==========================#
                         #=========================#
# Change YAD Terminal name to YesXR one's
# Gnome Terminal setup for Debugging and Testing purpose
#
# Constants declaration
#
# YesXR *CONTROL PANEL* --- YAD Main Window
#     YesXR Presentation & Commands description
#     Last action done / Last file used / Current MIDI/USB I/F selection
#     Note about ELECTONE / MDR / PC MIDI connections
#     YesXR main Menu with 6x Choices
#
# Scan silently MIDI/USB I/Fs -> .amidi.txt / .amidi2.txt
#
# IF [6- QUIT] Button
#     [QUIT] with TimeOut Progress Bar -- Yad window
#     Disconnect/Remove All ALSA MIDI connections
#     Delete .amidi.txt / .amidi2.txt
#     LEAVE YesXR script
#
#     ELIF (Test .amidi.txt)
#         [MIDI I/F Error] with TimeOut Progress Bar -- Yad window
#         If Empty = Error MSG 'NO I/F Detected!'
#         Return to Control Panel
#
#     ELIF [1- MIDI/USB I/Fs] Button
#         [MIDI/USB I/F Choice] -- Yad window
#         Scan silently MIDI/USB I/Fs -> .amidi.txt + .amidi2.txt
#             IF/FI (Test $ram/2_select) => If Empty = Error MSG 'NO I/F Selected!'
#         Return to Control Panel
#
#     ELIF (Test $ram_select)
#         [MIDI I/F Error] -- Yad window
#             IF/FI (Test $ram/2_file) => If Empty = Error MSG 'NO I/F Selected'
#         Return to Control Panel
#
#     ELIF [2- RECORD-eRAM] Button
#         [RECORD ELECTONE-to-PC] -- Yad window
#             IF/FI (Test $ram/2_file) => If Empty = Error MSG 'No File entered/selected'
#             Return to Control Panel
#         Request-ELECTONE-to-Send All-eRAM-Data
#         Reads-from-ALSA-RawMIDI port and Save-to-File
#         CountUp Progress Bar -- Yad window
#             IF/FI (Test $ram_size) => If size=O = Error message 'File empty'
#         Return to Control Panel
#
#     ELIF [3- SEND-xRAM] Button
#         [SEND PC-to-ELECTONE] -- Yad window
#             IF/FI (Test $ram_file) => If Empty = Error MSG 'No File selected'
#             IF/FI (Test $ram_size) => If size=O = Error message 'File empty'
#             Return to Control Panel
#         Request-Electone-to-Receive All-RAM Data
#         Writes-from-File to ALSA RawMIDI port
#         CountDown Progress Bar -- Yad window
#         Return to Control Panel
#
#     ELIF [4- BACKUP-mRAM] Button
#         [BACKUP MDR-to-PC] -- Yad window
#             IF/FI (Test $ram_file) => If Empty = Error MSG 'No File entered/selected'
#             Return to Control Panel
#         Request-MDR-to-Play All-mRAM Data for selected song nb
#         Reads-from-ALSA-RawMIDI port and Save-Backup-to-File
#         CountUp Progress Bar - Yad window
#             IF/FI (Test $ram_size) => If size=O = Error message 'File empty'
#         Return to Control Panel
#
#     ELIF [5- MIDI MONITORs] Button
#         [HEX or TEXT or CANCEL] -- Yad window
#             IF/FI (HEX) All-RAM MIDI DATA & MSG Monitoring -> Gnome Terminal
#             IF/FI (TEXT) Readable MIDI DATA & MSG Monitoring -> Gnome Terminal
#             IF/FI (CANCEL) -> Return to Control Panel
#         Return to Control Panel
#
#     ELSE (Oops!)
#         If NO [Action Button] was pressed or something went wrong
#         RETURN to YesXR Control Panel
#
# FI  End of YesXR Script
                               #================#
#==============================> FLOW CHART END <=============================#
#=============================================================================#

#=====================================#
# YesXR Script -- Main Variables:
#   $action / $ret  =>  YesXR MAIN Control Panel (Yad)
#   $action_mon / $ret_mon  =>  Monitors Control Panel
#   $ram_act / $ram_out  =>  Last Action done with outcome & displayed on CP
#   $ram_select / $Dir / $Device / $Name  =>  Midi/Usb I/F Selection & Filters
#   $ram_device  =>  Midi/Usb I/F 'hw:x,y,z' used in 'amidi'
#   $ram2_select / $Port / $Client_name / $Port_name  =>  Midi/Usb I/F Selection & Filters
#   $ram2_device  =>  Midi/Usb I/F 'hw:x,y,z' used in 'arecordmidi'
#   $ram_file / $ram_size / $ram_way  =>  File.ext, its Size & its Direction
#   $ram_time  =>  Time for File.ext to receive/sent eRAM/mRAM (Control Panel)
#   $yad_pid  =>  Current Yad Window Pid
#   $x / $y  =>  Pulsing bars values (Speed & Seconds) in REC-eRAM & BCKup-mRAM
#   $ram_pbar  =>  Timeout Pulsing bar (Seconds) in SEND-xRAM
#===========================#


#=============================================================================#
#>>>> >>>>>>>>>>>>>>>>>>>>>>> >> SCRIPT's START << <<<<<<<<<<<<<<<<<<<<<< <<<<#
                            #======================#

#======# Title bars Variables declaration #======#
run_file=$(basename YesXR*.sh)      # Get Script file name
yad_ver=$(yad --version)            # Get Yad version including GTK+ one
# Get Alsa-Utils version
alsa_utils=$(amidi --version | (grep -oP '(?<=amidi version )[\d.]+'))

# YesXR Debugging Terminal
title="Debugging Terminal" && echo -e '\033]2;'$title -- $run_file \
-- Yad $yad_ver - Alsa-utils $alsa_utils'\007' && echo -ne "\e[8;100;300t"
#-------------------------------------#

#======# Constants declaration #======#
Yes_Mod=YesXR                   # YesXR Version
Yes_Img=yesxr_logo-small.png    # YesXR Logo/Icon file
#-------------------------------------#

                #**********************************#
#===============# YesXR [CONTROL PANEL] YAD Window #===============#
                #**********************************#
action=$(yad \
    --title="$run_file  --  CONTROL PANEL  --  Yad $yad_ver / Alsa-utils $alsa_utils" \
    --width="965" height="555" --fixed --center --on-top --no-escape \
    --image="$Yes_Img" --window-icon="$Yes_Img" \
    --text="<b><span color='lime'>  *****  YAMAHA:  ELECTONE Organs -and- MDR Peripherals  /  MIDI:  All-RAM - Bulk Data Dump  /  PC:  GNU/Linux OS  *****</span></b>\n
<b><span color='lime'><u>'$Yes_Mod'  is  a  MIDI  SysEx  utility  dedicated  to  Yamaha  ELECTONE  /  MDR  'vintage'  Series,  allowing  you  to  easily</u>:</span></b>
  - Record / Save All-RAM Data from Yamaha Electone's RAMs to GNU/Linux PC's Disk.
  - Load / Send back these All-RAM Data from PC's Disk to Yamaha Electone MIDI organ.
  - Backup / Save All-RAM Data from Yamaha MDR's storage device (for the selected song) to PC's Disk.
  - Print from the MIDI-IN port MIDI Data either as Hex Bytes -or- as Readable Text to dedicated Monitors.
Compatible with Yamaha Electone CHX/HX/HS/US/HE/HC/EL Series (40+ Organ Models - from 1986 to 2001) and Yamaha MDR peripherals.
This is achieved by managing dedicated MIDI System Exclusive (SysEx) / Bulk Dump related Messages (F0h, 43h, ..., ..., ..., ..., ..., F7h).\n
<b><span color='lime'><u>QUICK  START</u>:  ELECTONE's / MDR's All-RAM Data  =  All REG, SEQ, FMP, VOICE ... USER Parameters</span>  (but No-Live Performance Data!)</b>
  <b>1-  <span color='yellow'>[MIDI/USB I/Fs]</span></b>  --  Select the 2x ALSA PORTs to be used by $Yes_Mod  -for-  All-RAM / HEX-Monitor and TEXT-Monitor Operations!
  <b>2-  <span color='red'>[RECORD-eRAM]</span></b>  --  Receive from ELECTONE's RAMs -and- Record All-RAM Data to PC's Disk.
  <b>3-  <span color='lime'>[SEND-xRAM]</span></b>  --  Read from PC's Disk -and- Send All-RAM (eRAM or mRAM) Data to ELECTONE's RAMs.
  <b>4-  <span color='magenta'>[BACKUP-mRAM]</span></b>  --  Receive from MDR's Storage -and- Record All-RAM Data to PC's Disk. Then you can use [SEND-xRAM] to ELECTONE.
  <b>5-  <span color='yellow'>[MIDI MONITORs]</span></b>  --  Display MIDI-IN Data as Hex bytes -or- Human readable Text (Note, Control Change, Program Change, etc.).\n
<b>- Last  ACTION:  <span color='yellow'>[ $ram_act ]</span>  =  <span color='orange'>$ram_out</span></b>\n
<b>- Last  <span color='orange'>$ram_way</span>  FILE:  <span color='cyan'>$ram_file</span>  of  <span color='yellow'>$ram_size bytes</span>  (~ $ram_time s)</b>\n
<b><span color='lime'><u>Current MIDI/USB I/Fs - ALSA Ports</u>:</span>  <i>For both Operations, an I/F must be selected even if using only one (i.e. could be the same I/F)!</i></b>
  <b>- For  ALL-RAM / HEX-Monitor  =  <span color='yellow'>$ram_select</span></b>    (i.e.  Direction  -  Device nb  -  Name)
  <b>- For  TEXT-Monitor  =  <span color='yellow'>$ram2_select</span></b>    (i.e.  Port nb  -  Client name  -  Port name)\n
<b><span color='lime'><u>NOTE</u>:</span>  Make sure to choose first an ALSA MIDI I/F  -then-  if needed a different one for ELECTONE / MDR / MONITOR operations.</b>
  - Yamaha Electone 'vintage' Series (CHX/HX/HS/US/HE/HC/EL)    ---- MIDI ---- INTERFACE-a ---- USB ----    Linux PC / $Yes_Mod (Yad)
  - Yamaha Music Disk Recorder (MDR-2/MDR-3/MDR-4/MDR-10)    ---- MIDI ---- INTERFACE-b ---- USB ----    Linux PC / $Yes_Mod (Yad)\n" \
    --expander="About $Yes_Mod © www.linuxrouen.fr - A free/libre software under GNU-GPL3" \
    --text-info <<< "https://www.linuxrouen.fr/wp/yamaha-electone-sysex-recorder-yesxr/
https://www.linuxrouen.fr/wp/contact/#form
https://www.gnu.org/licenses/gpl-3.0.en.html" --show-uri --uri-color="cyan" \
    --buttons-layout="center" \
    --button="<b><span color='yellow'> <u>MIDI/USB I/F</u></span></b>!search!FIRST: Select the ALSA Interfaces:21" \
    --button="<b><span color='red'> RECORD-eRAM</span></b>!record!RECORD All-eRAM from-ELECTONE-to-PC:22" \
    --button="<b><span color='lime'> SEND-xRAM</span></b>!player_play!SEND All-xRAM from-PC-to-ELECTONE:23" \
    --button="<b><span color='magenta'> BACKUP-mRAM</span></b>!media-floppy!BACKUP All-mRAM from-MDR-to-PC:24" \
    --button="<b><span color='yellow'> MIDI MONITOR</span></b>!computer-symbolic!Display MIDI-IN Data Traffic as HEX or TEXT:25" \
    --button="<b> -- QUIT --</b>!stop!LEAVE YesXR utility:26")
ret=$?
#-------------------------------------#

      #***********************************************************************#
#=====# Ports SILENTLY scanned for getting the 'Connected ALSA MIDI/USB I/Fs' #=====#
      #***********************************************************************#
aconnect -x     # For reliability purpose, Disconnect/Remove All ALSA MIDI connections, if any

## 'All-RAM' & 'HEX-Monitor' Operations
echo "" > .amidi.txt > /dev/null    # Clean the File
amidi -l >> .amidi.txt              # Append the Recognized I/Fs List to File
sed -i '1d' .amidi.txt              # Delete the First line (Header) from File and Update it

## 'TEXT-Monitor' Operation
echo "" > .amidi2.txt > /dev/null   # Clean the File
arecordmidi -l >> .amidi2.txt       # Append the recognized I/Fs List to File
sed -i '1d' .amidi2.txt             # Delete the first line (Header) from File and Update it
#-------------------------------------#

#****************************************************************#
#>>> BEGINNING of the MAIN 'IF / ELIF / ELSE / FI' Conditions <<<#
#****************************************************************#

           #**********************#
#==========# IF [--QUIT--] Button #==========#
           #**********************#
if [ "$ret" = "26" ]; then
    # [QUIT] Yad window with TimeOut Progress Bar
    yad \
    --title="QUIT  --  $Yes_Mod" \
    --width="500" --height="100" --fixed --center --on-top --no-escape \
    --image="face-smile" --timeout=2 --timeout-indicator=bottom \
    --button="<b> BYE - BYE</b>!stop" --window-icon="$Yes_Img" \
    --text-align=center --justify=center \
    --text="\nYO!  See you soon!"
    aconnect -x     # Disconnect/Remove All ALSA MIDI connections, if any
    rm .amidi.txt   # Delete MIDI/USB I/Fs hidden file (All-RAM & HEX-Monitor Operations)
    rm .amidi2.txt  # Delete MIDI/USB I/Fs hidden File (TEXT-Monitor Operation)
    exec "$1"   # LEAVE YesXR script
#-------------------------------------#

      #*********************************************************#
#=====# IF -NO- MIDI/USB I/F Detected then Display an ERROR MSG #=====#
      #*********************************************************#
elif [ ! -s .amidi.txt ]; then     # Check if the File is Empty!
    ## [MIDI I/F Error] Yad window with TimeOut Progress Bar
    yad \
    --title="MIDI/USB  INTERFACEs  --  $Yes_Mod" \
    --width="550" --height="170" --fixed --center --on-top --no-escape \
    --button="<b> Return to Control Panel</b>!yad" --window-icon="$Yes_Img" \
    --text-align=center --timeout=5 --timeout-indicator=bottom \
    --text="<b><span color='red'>E R R O R !\nNO  MIDI/USB  Interface  DETECTED!</span></b>" \
    --wrap --justify=center \
    --text-info <<< "
Please, Verify ALL your MIDI/USB connections
(awa if needed your legacy ALSA/USB drivers)!"
    # Export updated $variables to Control Panel
    export ram_act="MIDI/USB I/F Detection" ram_out="INTERFACE MISSING!"
    exec "$0"   # Return to YesXR Control Panel
#-------------------------------------#

           #**************************#
#==========# IF [MIDI/USB I/F] Button #==========#
           #**************************#
elif [ "$ret" = "21" ]; then
    #=====# MIDI/USB I/F Choice for 'ALL-RAM & HEX-Monitor'
    # Ports SILENTLY scanned for getting ALSA MIDI/USB I/Fs & Variables
    echo "" > .amidi.txt > /dev/null    # Clean the File
    amidi -l > /dev/null                # Print silently a List of All Hardware MIDI Ports
    amidi -l | sed '1d' >> .amidi.txt   # Append I/Fs List to File & Delete 1st line (Header)
    INPUT=.amidi.txt                    # Variable = Name of the File used by the 'read command'
    ## [MIDI/USB I/F Choice] Yad window: Read '.amidi.txt' Line by Line
    ram_select=$(while IFS=" " read -r Dir Device Name; do echo "$Dir" "$Device" "$Name";
    done < $INPUT | \
        yad \
        --title="*All-RAM & HEX-Monitor* MIDI/USB I/F Choice  --  $Yes_Mod" \
        --width="600" --height="220" --fixed --center --on-top --no-escape --window-icon="$Yes_Img" \
        --button="<b> Confirm I/F Selection</b>!checkbox!Did you make a Selection?" --text-align=center \
        --list --separator= --column="Dir - Device -- Name -----  (Available I/Fs)" \
        --text="<b><span color='lime'>SELECT a</span><span color='yellow'>  physical MIDI/USB I/F  </span><span color='lime'>for connecting\nYamaha ELECTONE  -or-  Yamaha MDR  --to--  $Yes_Mod</span></b>")
    # Save Selected $Device in $ram_device
    ram_device=$(echo "$ram_select" | (grep -oP '(?<=IO )hw:[\d,]+'))
    # Export $variables for later use in Backup/Record/Send Dialogs/Commands
    export ram_select=$ram_select ram_device=$ram_device
    #=====# MIDI/USB I/F Choice for 'TEXT-Monitor'
    # Ports SILENTLY scanned for getting ALSA MIDI/USB I/Fs & Variables
    echo "" > .amidi2.txt > /dev/null    # Clean the File
    arecordmidi -l > /dev/null           # Print silently a List of All Hardware MIDI Ports (TEXT-Monitor)
    arecordmidi -l | sed '1d' >> .amidi2.txt     # Append I/Fs List to File & Delete 1st line (Header)
    INPUT=.amidi2.txt                 # Variable = Name of the File used by the 'read command'
    ## [MIDI/USB I/F Choice] Yad window: Read '.amidi2.txt' Line by Line
    ram2_select=$(while IFS=" " read -r Port Client_name Port_name; do echo "$Port" "$Client_name" "$Port_name";
    done < $INPUT | \
        yad \
        --title="*TEXT-Monitor*  MIDI/USB I/F Choice  --  $Yes_Mod" \
        --width="550" --height="220" --fixed --center --on-top --no-escape --window-icon="$Yes_Img" \
        --button="<b> Confirm I/F Selection</b>!checkbox!Did you make a Selection?" --text-align=center \
        --list --separator= --column="Port - Client Name ---------- Port Name ----------  (Available SW / HW MIDI Ports)" \
        --text="<b><span color='lime'>SELECT a</span><span color='yellow'>  physical MIDI/USB I/F  </span><span color='lime'>for connecting\nYamaha ELECTONE  -or-  Yamaha MDR  --to--  $Yes_Mod</span></b>")
    ram2_device=$(echo "$ram2_select" | (grep -oP '[\d]+:[\d]'))   # Save Selected $Device in $perf_device
    # Export $variables for later use in Backup/Record/Send Dialogs/Commands
    export ram2_select=$ram2_select ram2_device=$ram2_device
    #=====# Detect if Both MIDI/USB I/Fs were well selected by the Performer
    if [ -z "$ram_select" ] || [ -z "$ram2_select" ]; then  # Check if one or both variables are Empty!
        ## [MIDI I/F Error] Yad window with TimeOut Progress Bar
        yad \
        --title="*All-RAM and/or HEX/TEXT-Monitors* I/Fs  --  $Yes_Mod" \
        --width="600" --height="170" --fixed --center --on-top --no-escape \
        --button="<b> Return to Control Panel</b>!yad" --window-icon="$Yes_Img" \
        --text-align=center --wrap --justify=center --timeout=5 --timeout-indicator=bottom \
        --text="<b><span color='red'>E R R O R !\nNO  MIDI/USB  Interface  SELECTED!</span></b>" \
        --text-info <<< "
Please, Click again on the [MIDI/USB I/F] button -and-
carefully SELECT both I/Fs for All-RAM -and- TEXT-Monitor!"
        # Export updated $variables to CP
        export ram_act="MIDI/USB I/Fs Selection" ram_out="NOT BOTH or NONE!"
        exec "$0"   # Return to YesXR Control Panel
    fi
    # Export updated $variables to Control Panel
    export ram_act="MIDI/USB I/Fs Selection" ram_out="OK for BOTH"
    exec "$0"   # Return to YesXR Control Panel
#-------------------------------------#

      #***********************************************************#
#=====# IF $RAM/$RAM2_SELECT are empty; then Display an ERROR MSG #=====#
      #***********************************************************#
elif [ -z "$ram_select" ] || [ -z "$ram2_select" ]; then    # Check if one or both variables are Empty!
    ## [MIDI I/Fs Error] Yad window with TimeOut Progress Bar
    yad \
    --title="*All-RAM and/or HEX/TEXT-Monitors* I/Fs  --  $Yes_Mod" \
    --width="600" --height="170" --fixed --center --on-top --no-escape \
    --button="<b> Return to Control Panel</b>!yad" --window-icon="$Yes_Img" \
    --text-align=center --timeout=5 --timeout-indicator=bottom \
    --text="<b><span color='red'>E R R O R !\nNO  MIDI/USB  Interface  SELECTED!</span></b>" \
    --wrap --justify=center \
    --text-info <<< "
Please, Click again on the [MIDI/USB I/F] button -and-
carefully SELECT both I/Fs for All-RAM -and- TEXT-Monitor!"
    # Export updated $variables to CP
    export ram_act="MIDI/USB I/Fs Selection" ram_out="NOT BOTH or NONE!"
    exec "$0"   # Return to YesXR Control Panel
#-------------------------------------#

           #*************************#
#==========# IF [RECORD-eRAM] Button #==========#
           #*************************#
elif [ "$ret" = "22" ]; then
    ## [RECORD ELECTONE-to-PC] Yad window: Performer Types/Chooses the File.ext to Save it to PC Disk => Variable
    ram_file=$(yad \
        --title="RECORD All-eRAM / ELECTONE-to-PC  --  $Yes_Mod" \
        --width="750" --height="450" --center --on-top --no-escape \
        --text-align=center --window-icon="$Yes_Img" \
        --button="<b> Confirm SAVE File / RECORD All-eRAM</b>!checkbox!Name.ext or Selection done?" \
        --file --save --confirm-overwrite="\nDo you really want to OverWrite it?" \
        --text="<b><span color='lime'>TYPE a new</span><span color='yellow'>  NAME.ext  </span><span color='lime'>-or-  SELECT a</span><span color='yellow'>  FILE.ext  </span><span color='lime'>(recorded with $Yes_Mod)</span></b>")
        # Test if a File has been entered/selected, if not Error message!
        if [ -z "$ram_file" ]; then
            ## [RECORD FILE Error] Yad window with TimeOut Progress Bar
            yad \
            --title="RECORD FILE  --  $Yes_Mod" \
            --width="550" --height="170" --fixed --center --on-top --no-escape \
            --button="<b> Return to Control Panel</b>!yad" --window-icon="$Yes_Img" \
            --text-align=center --wrap --justify=center --timeout=5 --timeout-indicator=bottom \
            --text="<b><span color='red'>E R R O R !\nNO FILE  was  Entered or Selected\!</span></b>" \
            --text-info <<< "
Please, Click again on the [RECORD-eRAM] button
and TYPE or SELECT an appropriate FILE.ext!"
    # Export updated $variables to CP
            export ram_act="RECORD All-eRAM / ELECTONE-to-PC" ram_out="NO FILE!" \
                   ram_way="failed" ram_file=" ? " ram_size=" ? " ram_time=" ? "
            exec "$0"   # Return to YesXR Control Panel
        fi
    ## [RECORD All-eRAM] Yad window: Displayed until the end of this process (= End of 'amidi -r' command)
    x=0     # Piping [while ... done] to 'yad' simulates 'stdout' activity for '--progress'
    while [ $x -le 2000 ];
        do x=$(( x + 1 )); y=$(( $x / 10 )); echo '$y';
        echo "# Please WAIT...   Work in progress...   $y seconds"; sleep 0.1;
    done | yad \
           --title="RECORDing All-eRAM  --  $Yes_Mod" \
           --width="550" --height="150" --fixed --center --on-top --no-escape --no-buttons \
           --text-align=center --wrap --justify=center --window-icon="$Yes_Img" \
           --text="\n<b><span color='lime'>$ram_file</span></b>\nRECORDing from 'ELECTONE Organ' can take a while!\n" \
           --progress --pulsate --progress-text="This window will auto-close..." &
    #------------------------------#
    #--Request-ELECTONE-to-Send All-eRAM-Data, Reads from ALSA RawMIDI port and Save-to-File
    amidi -p $ram_device -i 200 -S "F0 43 70 70 10 F7" -r "$ram_file" -t 1
    #------------------------------#
    sleep 0.5   # Security margin for properly closing Recording process
    yad_pid=$(echo $!); kill $yad_pid   # Save current Yad's Window Pid then Kill this process
    ram_size=$(wc -c < "$ram_file")     # Print the Bytes count of File to Variable
        ## Error message if File Size = 0!
        if [ "$ram_size" = "0" ]; then
            # Export updated $variables to CP
            export ram_act="RECORD All-eRAM / ELECTONE-to-PC" ram_out="FILE Empty!" \
                   ram_way="recorded/saved" ram_file=$ram_file ram_size=$ram_size ram_time="0"
            exec "$0"   # Return to YesXR Control Panel
        fi
    # Export updated $variables to CP
    export ram_act="RECORD All-eRAM / ELECTONE-to-PC" ram_out="OK" ram_way="recorded/saved" \
           ram_file=$ram_file ram_size=$ram_size ram_time=$(( $ram_size / 3125 ))
    exec "$0"   # Return to YesXR Control Panel
#-------------------------------------#

           #***********************#
#==========# IF [SEND-xRAM] Button #==========#
           #***********************#
elif [ "$ret" = "23" ]; then
    ## [SEND PC-to-ELECTONE] Yad window: Performer chooses the File.ext to Send to Electone => Variable
    ram_file=$(yad \
        --title="SEND All-xRAM / PC-to-ELECTONE  --  $Yes_Mod" \
        --width="750" --height="450" --center --on-top --no-escape \
        --text-align=center --window-icon="$Yes_Img" \
        --file --button="<b> Confirm LOAD File / SEND All-xRAM</b>!checkbox!Did you make a Selection?" \
        --text="<b><span color='lime'>SELECT an existing</span><span color='yellow'>  FILE.ext  </span><span color='lime'>(which was recorded with $Yes_Mod)</span></b>")
        # Test if a File has been selected or not! (but it's different from [Record/Backup] because a selection is done by default!)
        if [ ! -f "$ram_file" ]; then
            ## [SEND FILE Error] Yad window with TimeOut Progress Bar
            yad \
            --title="SEND FILE  --  $Yes_Mod" \
            --width="550" --height="170" --fixed --center --on-top --no-escape \
            --button="<b> Return to Control Panel</b>!yad" --window-icon="$Yes_Img" \
            --text-align=center --wrap --justify=center --timeout=5 --timeout-indicator=bottom \
            --text="<b><span color='red'>E R R O R !\nNO FILE  was  Selected\!</span></b>" \
            --text-info <<< "
Please, Click again on the [SEND-xRAM] button
and SELECT an appropriate FILE.ext!"
            # Export updated $variables to CP
            export ram_act="SEND All-xRAM / PC-to-ELECTONE" ram_out="NO FILE!" \
                   ram_way="failed" ram_file=" ? " ram_size=" ? " ram_time=" ? "
            exec "$0"   # Return to YesXR Control Panel
        fi
    # Prepare File & Progress bar Variables
    ram_size=$(wc -c < "$ram_file")       # Print the Bytes count of File to Variable
        ## Error message if File Size = 0!
        if [ "$ram_size" = "0" ]; then
            # Export updated $variables to CP
            export ram_act="SEND All-xRAM / PC-to-ELECTONE" ram_out="FILE Empty!" \
                   ram_way="loaded/sent" ram_file=$ram_file ram_size=$ram_size ram_time="0"
            exec "$0"   # Return to YesXR Control Panel
        fi
    let ram_pbar=$(( $ram_size / 3125 + 1 ))    # Compute Progress bar Timing value Variable
    #------------------------------#
    #--Note: 'amidi' '--send' & --'send-hex' can't be used at the same time on the same cd line!
        # Request-Electone-to-Receive All-RAM Data
    amidi -p $ram_device -i 200 -S "F0 43 70 70 20 F7"
    amidi -p $ram_device -s "$ram_file" &
        # Write-from-File to ALSA RawMIDI port
    #------------------------------#
    ## [SENDing All-xRAM] Yad window: with Countdown Progress Bar (relative to File.ext size)
    yad \
    --title="SENDing All-xRAM  --  $Yes_Mod" \
    --width="550" --height="150" --fixed --center --on-top --no-escape --no-buttons \
    --text-align=center --wrap --justify=center --window-icon="$Yes_Img" \
    --timeout=$ram_pbar --timeout-indicator=bottom \
    --text="\n<b><span color='lime'>$ram_file</span>  =  <span color='yellow'>$ram_size bytes</span></b>\n
SENDing to 'ELECTONE Organ' can take a while!\n
Please WAIT... until this window auto-closes..." &
    wait        # Before continuing, Wait for previous 2 parallel '&' commands to finish
    sleep 0.5   # Security margin for properly closing Sending process
    # Export updated $variables to CP
    export ram_act="SEND All-xRAM / PC-to-ELECTONE" ram_out="OK" ram_way="loaded/sent" \
           ram_file=$ram_file ram_size=$ram_size ram_time=$(( $ram_size / 3125 ))
    exec "$0"   # Return to YesXR Control Panel
#-------------------------------------#

           #*************************#
#==========# IF [BACKUP-mRAM] Button #==========#
           #*************************#
elif [ "$ret" = "24" ]; then
    ## [BACKUP MDR-to-PC] Yad window: Performer Types/Chooses the File.ext to Save it to PC Disk => Variable
    ram_file=$(yad \
        --title="BACKUP All-mRAM / MDR-to-PC  --  $Yes_Mod" \
        --width="750" --height="450" --center --on-top --no-escape \
        --text-align=center --window-icon="$Yes_Img" \
        --button="<b> Confirm SAVE File / BACKUP All-mRAM</b>!checkbox!Name.ext or Selection done?" \
        --file --save --confirm-overwrite="\nDo you really want to OverWrite IT?" \
        --text="<b><span color='lime'>TYPE a new</span><span color='yellow'>  NAME.ext  </span><span color='lime'>-or-  SELECT a</span><span color='yellow'>  FILE.ext  </span><span color='lime'>(recorded with $Yes_Mod)</span></b>")
        # Test if a File has been entered/selected, if not Error message!
        if [ -z "$ram_file" ]; then
            ## [BACKUP FILE Error] Yad window with TimeOut Progress Bar
            yad \
            --title="BACKUP FILE  --  $Yes_Mod" \
            --width="550" --height="170" --fixed --center --on-top --no-escape \
            --button="<b> Return to Control Panel</b>!yad" --window-icon="$Yes_Img" \
            --text-align=center --wrap --justify=center --timeout=5 --timeout-indicator=bottom \
            --text="<b><span color='red'>E R R O R !\nNO FILE  was  Entered or Selected\!</span></b>" \
            --text-info <<< "
Please, Click again on the [BACKUP-mRAM] button
and TYPE or SELECT an appropriate FILE.ext!"
            # Export updated $variables to CP
            export ram_act="BACKUP All-mRAM / MDR-to-PC" ram_out="NO FILE!" \
                   ram_way="failed" ram_file=" ? " ram_size=" ? " ram_time=" ? "
            exec "$0"   # Return to YesXR Control Panel
        fi
    ## [BACKUPing All-mRAM] Yad window: Displayed until the end of this process (= End of 'amidi -r')
    x=0     # Piping [while ... done] to 'yad' simulates 'stdout' activity for --progress
    while [ $x -le 2000 ];
        do x=$(( x + 1 )); y=$(( $x / 10 )); echo '$y';
        echo "# Please WAIT...   Work in progress...   $y seconds"; sleep 0.1;
    done | yad \
           --title="BACKUPing All-mRAM  --  $Yes_Mod" \
           --width="550" --height="150" --fixed --center --on-top --no-escape --no-buttons \
           --text-align=center --wrap --justify=center --window-icon="$Yes_Img" \
           --text="\n<b><span color='lime'>$ram_file</span></b>\nBACKUPing from 'MDR Peripheral' can take a while!\n" \
           --progress --pulsate --progress-text="WAIT... until this window auto-closes." &
    #------------------------------#
    #--Request-MDR-to-Play ONLY All-mRAM Data of the selected song nb,
        #  Reads from ALSA RawMIDI port and Save mBackup to File
    amidi -p "$ram_device" -r "$ram_file" -t 2 -i 200 \
          -S "F0 43 70 70 72 00 7F F7" -S "F0 43 70 70 72 00 00 F7" \
          -S "F0 43 70 70 72 01 7F F7" -S "F0 43 70 70 72 01 00 F7"
        #  SysEx: 1-MDR starts sending All-mRAM Data & 2-MDR stops when All-mRAM transmitted
    #------------------------------#
    sleep 1     # Security margin for properly closing mRecording process
    yad_pid=$(echo $!); kill $yad_pid   # Save current Yad's Window Pid then Kill this process
    ram_size=$(wc -c < "$ram_file")     # Print the Bytes count of File to Variable
        ## Error message if File Size = 0!
        if [ "$ram_size" = "0" ]; then
            # Export updated $variables to CP
            export ram_act="BACKUP All-mRAM / MDR-to-PC" ram_out="FILE Empty!" \
                   ram_way="backuped/saved" ram_file=$ram_file ram_size=$ram_size ram_time="0"
            exec "$0"   # Return to YesXR Control Panel
        fi
    # Export updated $variables to CP
    export ram_act="BACKUP All-mRAM / MDR-to-PC" ram_out="OK" ram_way="backuped/saved" \
           ram_file=$ram_file ram_size=$ram_size ram_time=$(( $ram_size / 3125 ))
    exec "$0"   # Return to YesXR Control Panel
#-------------------------------------#

      #***************************#
#=====# IF [MIDI MONITORs] Button #=====#
      #***************************#
elif [ "$ret" = "25" ]; then
    #=====# MONITORs "Control Panel" Yad window
    action_mon=$(yad \
        --title="$Yes_Mod  --  MIDI TRAFFIC MONITOR Choice" \
        --width="550" height="150" --fixed --center --on-top --no-escape --window-icon="$Yes_Img" \
        --text-align=center --justify=center \
        --text="<b><u>S E L E C T</u>:\n
<span color='red'>* HEX *</span>  for  MIDI SYSEX Data and Others  (from ALSA Port: $ram_device)\n
- or -\n
<span color='lime'>* TEXT *</span>  for  READABLE MIDI Messages and Data  (from ALSA Port: $ram2_device)</b>\n" \
        --buttons-layout="center" \
        --button="<b> HEX  (like All-RAM)</b>!flag-red!Display MIDI-IN TRAFFIC as HEX Bytes:31" \
        --button="<b> TEXT  (like Notes, PC, CC)</b>!flag-green!Display MIDI-IN TRAFFIC as Readable TEXT:32" \
        --button="<b> --CANCEL--</b>!yad!Return to Control Panel:33")
    ret_mon=$?
    #=====# IF [HEX] Button
    if [ "$ret_mon" = "31" ]; then
        #=====# Open a Terminal & Execute inside the 'amidi' command for Monitoring MIDI DATA & MSG.
        gnome-terminal --title="$Yes_Mod  --  *HEX* MIDI TRAFFIC / MONITORing Terminal" \
        --geometry=90x30+0+0 --hide-menubar \
        -- bash -c 'echo "-------------------------"; echo "- ALSA MIDI/USB INTERFACE = $ram_select"; \
        echo "-------------------------"; echo "-> Press [CTRL] + [C] for closing this MONITORing Terminal!"; \
        echo "-------------------------"; \
        echo "... Waiting to RECEIVE MIDI-IN Data/Messages as HEXadecimal Bytes..."; \
        amidi -p $ram_device -d'
        #--'amidi' prints received raw MIDI Data/Messages as hexadecimal bytes
        #  'Clock' & 'Active Sensing' messages are automatically filtered.
        #------------------------------#
        #=====#
        # Export $variables to update Control Panel
        export ram_act="MIDI TRAFFIC / HEX Bytes MONITORing" ram_out="OK"
        exec "$0"   # Return to YesMDR's Control Panel
    #=====# IF [TEXT] Button
    elif [ "$ret_mon" = "32" ]; then
        #=====# Open a New Terminal, execute inside the 'aseqdump' command for Monitoring ALL MIDI MSG
        gnome-terminal --title="$Yes_Mod  --  *TEXT* MIDI TRAFFIC / MONITORing Terminal" \
        --geometry=90x30+0+0 --hide-menubar \
        -- bash -c 'echo "-------------------------"; echo "ALSA MIDI/USB INTERFACE = $ram2_select"; \
        echo "-------------------------"; echo "-> Press [CTRL] + [C] for closing this MONITORing Terminal!"; \
        echo "-------------------------"; \
        echo "... Waiting to RECEIVE MIDI-IN Data/Messages as Human Readable TEXT..."; \
        aseqdump -p $ram2_device | grep -v "Clock\|Active Sensing"'
        #--'aseqdump' prints received raw MIDI Data/Messages as Text
        #   with fitering unnecessary 'Clock' & 'Active Sensing' messages.
        #=====#
        # Export $variables to update Control Panel
        export ram_act="MIDI TRAFFIC - Readable TEXT MONITORing" ram_out="OK"
        exec "$0"   # Return to YesMDR's Control Panel
    #=====# IF [CANCEL] Button
    elif [ "$ret_mon" = "33" ]; then
        # Export $variables to update Control Panel
        export ram_act="MIDI TRAFFIC MONITORing" ram_out="CANCELED"
        exec "$0"   # Return to YesMDR's Control Panel
    #=====# IF NONE of above 3x Choices
    else
        export ram_act="MIDI TRAFFIC MONITORing" ram_out="NO CHOICE was done"
        exec "$0"   # Return to YesMDR's Control Panel
    #=====#
    fi
#-------------------------------------#

      #*****************************#
#=====# IF NONE of above 6x CHOICES #=====#
      #*****************************#
else
    export ram_act="YesXR CONTROL PANEL" ram_out="OOPS!  NO CHOICE was done!"
    exec "$0"   # Return to YesMDR's Control Panel
#-------------------------------------#

#**********************************************************#
#>>> END of the MAIN 'IF / ELIF / ELSE / FI' Conditions <<<#
#**********************************************************#
fi

                             #====================#
#>>>> >>>>>>>>>>>>>>>>>>>>>>>> >> SCRIPT's END << <<<<<<<<<<<<<<<<<<<<<<< <<<<#
#=============================================================================#

